import platform

if platform.system() == "Windows":
    from .windows import get_current_song
elif platform.system() == "Linux":
    from .linux import get_current_song
else:
    raise RuntimeError("Unsupported operating system")

__all__ = ["get_current_song"]


import re
from unidecode import unidecode

import webbrowser


def remove_features(title_song: str) -> str:
    return re.sub(
        r"\s*\((feat|ft|with)[^)]*\)",
        "",
        title_song,
        flags=re.IGNORECASE
    ).strip()


def generate_genius_slug(artist: str, title: str) -> str:
    title = remove_features(title)

    text = f"{artist} {title}"
    
    text = text.replace("'", "")

    text = text.replace('"', "")
    
    text = text.replace("&", "and")

    text = unidecode(text)

    # Replace non-alphanumeric with hyphen
    text = re.sub(r"[^a-zA-Z0-9]+", "-", text)

    # Collapse multiple hyphens
    text = re.sub(r"-+", "-", text)

    return text.strip("-") + "-lyrics"


def open_genius(artist: str, title: str):
    url = f"https://genius.com/{generate_genius_slug(artist, title)}"
    print(f'Opening {url}')
    webbrowser.open(url)


import dbus


def get_current_song():
    try:
        session_bus = dbus.SessionBus()
        spotify_bus = session_bus.get_object(
            "org.mpris.MediaPlayer2.spotify",
            "/org/mpris/MediaPlayer2"
        )

        interface = dbus.Interface(
            spotify_bus,
            "org.freedesktop.DBus.Properties"
        )

        metadata = interface.Get(
            "org.mpris.MediaPlayer2.Player",
            "Metadata"
        )

        artist = str(metadata["xesam:artist"][0])
        title = str(metadata["xesam:title"])

        return artist, title

    except Exception:
        return None, None


import ctypes
import win32gui
import win32process
import psutil


GetWindowText = ctypes.windll.user32.GetWindowTextW
GetWindowTextLength = ctypes.windll.user32.GetWindowTextLengthW
IsWindowVisible = ctypes.windll.user32.IsWindowVisible


def _get_spotify_pids():
    spotify_pids = []

    for proc in psutil.process_iter(["name"]):
        try:
            if proc.info["name"] and "Spotify.exe" in proc.info["name"]:
                spotify_pids.append(proc.pid)
        except (psutil.NoSuchProcess, psutil.AccessDenied):
            continue

    return spotify_pids


def _get_hwnds_for_pid(pid):
    hwnds = []

    def callback(hwnd, _):
        _, found_pid = win32process.GetWindowThreadProcessId(hwnd)
        if found_pid == pid:
            hwnds.append(hwnd)
        return True

    win32gui.EnumWindows(callback, None)
    return hwnds


def _get_window_title(hwnd):
    length = GetWindowTextLength(hwnd)
    buff = ctypes.create_unicode_buffer(length + 1)
    GetWindowText(hwnd, buff, length + 1)
    return buff.value


def get_current_song():
    """
    Returns (artist, title) if Spotify is playing.
    Returns (None, None) otherwise.
    """

    pids = _get_spotify_pids()

    for pid in pids:
        hwnds = _get_hwnds_for_pid(pid)

        for hwnd in hwnds:
            if IsWindowVisible(hwnd):
                title = _get_window_title(hwnd)

                if " - " not in title:
                    continue

                parts = title.split(" - ")

                if len(parts) != 2:
                    continue

                artist, song = parts
                return artist.strip(), song.strip()

    return None, None


import time
from spotify_genius.platforms import get_current_song
from spotify_genius.core import open_genius

def run():
    previous = None

    while True:
        artist, title = get_current_song()

        if not artist or not title:
            time.sleep(2)
            continue

        current = (artist, title)

        if current != previous:
            previous = current
            print(f'Now playing: {artist} - {title}')

            open_genius(artist, title)

        time.sleep(2)

if __name__ == "__main__":
    run()